@@ -124,6 +124,7 @@ static guint signals[LAST_SIGNAL];
 static void meta_cursor_tracker_set_crtc_has_hw_cursor (MetaCursorTracker *tracker,
                                                         MetaCRTC          *crtc,
                                                         gboolean           has_hw_cursor);
+static void sync_cursor (MetaCursorTracker *tracker);
 
 static MetaCursorReference *
 meta_cursor_reference_ref (MetaCursorReference *self)
@@ -675,6 +676,7 @@ set_window_cursor (MetaCursorTracker   *tracker,
   if (cursor)
     tracker->window_cursor = meta_cursor_reference_ref (cursor);
   tracker->has_window_cursor = has_cursor;
+  sync_cursor (tracker);
 }
 
 gboolean
@@ -694,7 +696,6 @@ meta_cursor_tracker_handle_xevent (MetaCursorTracker *tracker,
     return FALSE;
 
   set_window_cursor (tracker, FALSE, NULL);
-  g_signal_emit (tracker, signals[CURSOR_CHANGED], 0);
 
   return TRUE;
 }
@@ -839,6 +840,7 @@ meta_cursor_tracker_set_grab_cursor (MetaCursorTracker *tracker,
   g_clear_pointer (&tracker->grab_cursor, meta_cursor_reference_unref);
   if (cursor != META_CURSOR_DEFAULT)
     tracker->grab_cursor = ensure_wayland_cursor (tracker, cursor);
+  sync_cursor (tracker);
 }
 
 void
@@ -882,6 +884,7 @@ meta_cursor_tracker_set_root_cursor (MetaCursorTracker *tracker,
     {
       g_clear_pointer (&tracker->root_cursor, meta_cursor_reference_unref);
       tracker->root_cursor = ensure_wayland_cursor (tracker, cursor);
+      sync_cursor (tracker);
     }
 }
 
@@ -966,12 +969,15 @@ sync_displayed_cursor (MetaCursorTracker *tracker)
   if (displayed_cursor)
     tracker->displayed_cursor = meta_cursor_reference_ref (displayed_cursor);
 
-  if (displayed_cursor)
-    cogl_pipeline_set_layer_texture (tracker->pipeline, 0, COGL_TEXTURE (displayed_cursor->texture));
-  else
-    cogl_pipeline_set_layer_texture (tracker->pipeline, 0, NULL);
+  if (meta_is_wayland_compositor ())
+    {
+      if (displayed_cursor)
+        cogl_pipeline_set_layer_texture (tracker->pipeline, 0, COGL_TEXTURE (displayed_cursor->texture));
+      else
+        cogl_pipeline_set_layer_texture (tracker->pipeline, 0, NULL);
 
-  update_hw_cursor (tracker);
+      update_hw_cursor (tracker);
+    }
 
   g_signal_emit (tracker, signals[CURSOR_CHANGED], 0);
 }
@@ -1030,10 +1036,13 @@ sync_cursor (MetaCursorTracker *tracker)
       tracker->current_rect.height = 0;
     }
 
-  if (tracker->has_hw_cursor)
-    move_hw_cursor (tracker);
-  else
-    meta_cursor_tracker_queue_redraw (tracker);
+  if (meta_is_wayland_compositor ())
+    {
+      if (tracker->has_hw_cursor)
+        move_hw_cursor (tracker);
+      else
+        meta_cursor_tracker_queue_redraw (tracker);
+    }
 }
 
 void
