@@ -92,16 +92,6 @@ meta_stack_new (MetaScreen *screen)
 static void
 free_last_all_root_children_stacked_cache (MetaStack *stack)
 {
-  unsigned int i;
-
-  for (i = 0; i < stack->last_all_root_children_stacked->len; i++)
-    {
-      MetaStackWindow *window = &g_array_index (stack->last_all_root_children_stacked, MetaStackWindow, i);
-      if (window->any.type == META_WINDOW_CLIENT_TYPE_WAYLAND)
-        g_object_remove_weak_pointer (G_OBJECT (window->wayland.meta_window),
-                                      (gpointer *)&window->wayland.meta_window);
-    }
-
   g_array_free (stack->last_all_root_children_stacked, TRUE);
   stack->last_all_root_children_stacked = NULL;
 }
@@ -1337,17 +1327,6 @@ stack_sync_to_xserver (MetaStack *stack)
       /* build XRestackWindows() array from top to bottom */
       if (w->client_type == META_WINDOW_CLIENT_TYPE_X11)
         g_array_append_val (x11_root_children_stacked, top_level_window);
-      else
-        {
-          MetaStackWindow *new;
-
-          /* So we can determine later if a cached stack window is
-           * stale because the corresponding window has been freed we
-           * associate a weak pointer with the new window. */
-          new = &g_array_index (all_root_children_stacked, MetaStackWindow, all_root_children_stacked->len - 1);
-          g_object_add_weak_pointer (G_OBJECT (new->wayland.meta_window),
-                                     (gpointer *)&new->wayland.meta_window);
-        }
     }
 
   meta_topic (META_DEBUG_STACK, "\n");
@@ -1705,7 +1684,7 @@ window_contains_point (MetaWindow *window,
 {
   MetaRectangle rect;
 
-  meta_window_get_outer_rect (window, &rect);
+  meta_window_get_frame_rect (window, &rect);
 
   return POINT_IN_RECT (root_x, root_y, rect);
 }
