@@ -102,7 +102,7 @@ struct _MetaWindow
   
   MetaWindowType type;
   Atom type_atom;
-  
+
   /* NOTE these five are not in UTF-8, we just treat them as random
    * binary data
    */
@@ -351,6 +351,10 @@ struct _MetaWindow
   /* whether or not the window is from a program running on another machine */
   guint is_remote : 1;
 
+  /* Used for Wayland -- surfaces can behave as if they were unmapped if
+   * they have a NULL buffer attached... */
+  guint surface_mapped;
+
   /* if non-NULL, the bounds of the window frame */
   cairo_region_t *frame_bounds;
 
@@ -363,6 +367,9 @@ struct _MetaWindow
   /* the input shape region for picking */
   cairo_region_t *input_region;
 
+  /* _NET_WM_WINDOW_OPACITY */
+  guint opacity;
+
   /* if TRUE, the we have the new form of sync request counter which
    * also handles application frames */
   guint extended_sync_request_counter : 1;
@@ -462,6 +469,8 @@ struct _MetaWindow
 
   /* Bypass compositor hints */
   guint bypass_compositor;
+
+  GSList *pending_pings;
 };
 
 struct _MetaWindowClass
@@ -508,8 +517,6 @@ MetaWindow* meta_window_new_with_attrs     (MetaDisplay       *display,
                                             MetaCompEffect     effect,
                                             XWindowAttributes *attrs);
 MetaWindow *meta_window_new_for_wayland    (MetaDisplay        *display,
-                                            int                 width,
-                                            int                 height,
                                             MetaWaylandSurface *surface);
 void        meta_window_unmanage           (MetaWindow  *window,
                                             guint32      timestamp);
@@ -644,8 +651,10 @@ void meta_window_update_sync_request_counter (MetaWindow *window,
                                               gint64      new_counter_value);
 #endif /* HAVE_XSYNC */
 
-void meta_window_handle_mouse_grab_op_event (MetaWindow         *window,
-                                             const ClutterEvent *event);
+void meta_window_handle_mouse_grab_op_event  (MetaWindow         *window,
+                                              const ClutterEvent *event);
+void meta_window_handle_mouse_grab_op_xevent (MetaWindow         *window,
+                                              XIDeviceEvent      *xevent);
 
 GList* meta_window_get_workspaces (MetaWindow *window);
 
@@ -709,16 +718,8 @@ void meta_window_compute_tile_match (MetaWindow *window);
 
 gboolean meta_window_updates_are_frozen (MetaWindow *window);
 
-void meta_window_set_opaque_region        (MetaWindow     *window,
-                                           cairo_region_t *region);
 void meta_window_update_opaque_region_x11 (MetaWindow *window);
-
-void meta_window_set_input_region         (MetaWindow     *window,
-                                           cairo_region_t *region);
 void meta_window_update_input_region_x11  (MetaWindow *window);
-
-void meta_window_set_shape_region         (MetaWindow     *window,
-                                           cairo_region_t *region);
 void meta_window_update_shape_region_x11  (MetaWindow *window);
 
 void meta_window_set_title                (MetaWindow *window,
@@ -737,9 +738,28 @@ void meta_window_set_gtk_dbus_properties  (MetaWindow *window,
 void meta_window_set_transient_for        (MetaWindow *window,
                                            MetaWindow *parent);
 
+void meta_window_set_opacity              (MetaWindow *window,
+                                           guint       opacity);
+
 void meta_window_handle_enter (MetaWindow  *window,
                                guint32      timestamp,
                                guint        root_x,
                                guint        root_y);
 
+void meta_window_set_surface_mapped (MetaWindow *window,
+                                     gboolean    surface_mapped);
+
+typedef void (* MetaWindowPingFunc) (MetaWindow  *window,
+				     guint32      timestamp,
+				     gpointer     user_data);
+
+void meta_window_ping (MetaWindow         *window,
+                       guint32             timestamp,
+                       MetaWindowPingFunc  ping_reply_func,
+                       MetaWindowPingFunc  ping_timeout_func,
+                       void               *user_data);
+void meta_window_pong (MetaWindow *window,
+                       guint32     timestamp);
+Window meta_window_get_toplevel_xwindow (MetaWindow *window);
+
 #endif
