@@ -318,8 +318,11 @@ meta_wayland_pointer_set_focus (MetaWaylandPointer *pointer,
   struct wl_resource *resource, *kr;
   uint32_t serial;
 
+  if (pointer->focus == surface && pointer->focus_resource != NULL)
+    return;
+
   resource = pointer->focus_resource;
-  if (resource && pointer->focus != surface)
+  if (resource && pointer->focus->resource)
     {
       struct wl_client *client = wl_resource_get_client (resource);
       struct wl_display *display = wl_client_get_display (client);
@@ -329,8 +332,7 @@ meta_wayland_pointer_set_focus (MetaWaylandPointer *pointer,
     }
 
   resource = find_resource_for_surface (&pointer->resource_list, surface);
-  if (resource &&
-      (pointer->focus != surface || pointer->focus_resource != resource))
+  if (resource)
     {
       struct wl_client *client = wl_resource_get_client (resource);
       struct wl_display *display = wl_client_get_display (client);
@@ -446,24 +448,6 @@ meta_wayland_pointer_end_modal (MetaWaylandPointer *pointer)
   g_slice_free (MetaWaylandPointerGrab, grab);
 }
 
-/* Called when the focused resource is destroyed */
-void
-meta_wayland_pointer_destroy_focus (MetaWaylandPointer *pointer)
-{
-  if (pointer->grab == &pointer->default_grab)
-    {
-      /* The surface was destroyed, but had the implicit pointer grab.
-         Bypass the grab interface. */
-      g_assert (pointer->button_count > 0);
-
-      /* Note: we focus the NULL interface, not the current one, because
-	 we have button down, and the clients would be confused if the
-	 pointer enters the surface.
-      */
-      meta_wayland_pointer_set_focus (pointer, NULL);
-    }
-}
-
 typedef struct {
   MetaWaylandPointerGrab  generic;
 
@@ -488,7 +472,7 @@ popup_grab_focus (MetaWaylandPointerGrab *grab,
 
   /* Popup grabs are in owner-events mode (ie, events for the same client
      are reported as normal) */
-  if (wl_resource_get_client (surface->resource) == popup_grab->grab_client)
+  if (surface && wl_resource_get_client (surface->resource) == popup_grab->grab_client)
     default_grab_focus (grab, surface, event);
   else
     meta_wayland_pointer_set_focus (grab->pointer, NULL);
@@ -538,14 +522,14 @@ meta_wayland_pointer_end_popup_grab (MetaWaylandPointer *pointer)
 
   wl_list_for_each_safe (popup, tmp, &popup_grab->all_popups, link)
     {
-      MetaWaylandSurfaceExtension *shell_surface = popup->surface->xdg_surface;
-      struct wl_client *client = wl_resource_get_client (shell_surface->resource);
+      MetaWaylandSurfaceExtension *xdg_popup = &popup->surface->xdg_popup;
+      struct wl_client *client = wl_resource_get_client (xdg_popup->resource);
       struct wl_display *display = wl_client_get_display (client);
       uint32_t serial;
 
       serial = wl_display_next_serial (display);
 
-      xdg_popup_send_popup_done (shell_surface->resource, serial);
+      xdg_popup_send_popup_done (xdg_popup->resource, serial);
       wl_list_remove (&popup->surface_destroy_listener.link);
       wl_list_remove (&popup->link);
       g_slice_free (MetaWaylandPopup, popup);
@@ -607,7 +591,7 @@ meta_wayland_pointer_start_popup_grab (MetaWaylandPointer *pointer,
   popup->grab = grab;
   popup->surface = surface;
   popup->surface_destroy_listener.notify = on_popup_surface_destroy;
-  wl_resource_add_destroy_listener (surface->resource, &popup->surface_destroy_listener);
+  wl_resource_add_destroy_listener (surface->xdg_popup.resource, &popup->surface_destroy_listener);
 
   wl_list_insert (&grab->all_popups, &popup->link);
       
