@@ -77,6 +77,10 @@ MetaWaylandSeat *
 meta_wayland_seat_new (struct wl_display *display,
 		       gboolean           is_native);
 
+void
+meta_wayland_seat_update_pointer (MetaWaylandSeat *seat,
+                                  const ClutterEvent *event);
+
 gboolean
 meta_wayland_seat_handle_event (MetaWaylandSeat *seat,
                                 const ClutterEvent *event);
