@@ -44,7 +44,7 @@
 #include "meta-wayland-private.h"
 #include "meta-xwayland-private.h"
 #include "meta-wayland-stage.h"
-#include "meta-window-actor-private.h"
+#include "meta-surface-actor.h"
 #include "meta-wayland-seat.h"
 #include "meta-wayland-keyboard.h"
 #include "meta-wayland-pointer.h"
@@ -63,40 +63,18 @@ static void
 surface_process_damage (MetaWaylandSurface *surface,
                         cairo_region_t *region)
 {
-  g_assert (surface->window);
+  int i, n_rectangles = cairo_region_num_rectangles (region);
 
-  if (surface->buffer_ref.buffer)
+  for (i = 0; i < n_rectangles; i++)
     {
-      MetaWindowActor *window_actor =
-        META_WINDOW_ACTOR (meta_window_get_compositor_private (surface->window));
-      MetaRectangle rect;
-      cairo_rectangle_int_t cairo_rect;
-
-      meta_window_get_input_rect (surface->window, &rect);
-      cairo_rect.x = 0;
-      cairo_rect.y = 0;
-      cairo_rect.width = rect.width;
-      cairo_rect.height = rect.height;
-
-      cairo_region_intersect_rectangle (region, &cairo_rect);
-
-      if (window_actor)
-        {
-          int i, n_rectangles = cairo_region_num_rectangles (region);
-
-          for (i = 0; i < n_rectangles; i++)
-            {
-              cairo_rectangle_int_t rectangle;
-
-              cairo_region_get_rectangle (region, i, &rectangle);
-
-              meta_window_actor_process_wayland_damage (window_actor,
-                                                        rectangle.x,
-                                                        rectangle.y,
-                                                        rectangle.width,
-                                                        rectangle.height);
-            }
-        }
+      cairo_rectangle_int_t rect;
+      cairo_region_get_rectangle (region, i, &rect);
+      meta_surface_actor_damage_area (surface->surface_actor,
+                                      rect.x,
+                                      rect.y,
+                                      rect.width,
+                                      rect.height,
+                                      NULL);
     }
 }
 
@@ -236,30 +214,6 @@ empty_region (cairo_region_t *region)
   cairo_region_intersect_rectangle (region, &rectangle);
 }
 
-static gboolean
-surface_wants_window (MetaWaylandSurface *surface)
-{
-  return (surface->xdg_surface != NULL);
-}
-
-static void
-surface_ensure_window (MetaWaylandSurface *surface)
-{
-  MetaDisplay *display = meta_get_display ();
-  int width, height;
-
-  if (surface->window)
-    return;
-
-  if (!surface_wants_window (surface))
-    return;
-
-  width = surface->buffer_ref.buffer->width;
-  height = surface->buffer_ref.buffer->height;
-
-  surface->window = meta_window_new_for_wayland (display, width, height, surface);
-}
-
 static void
 ensure_buffer_texture (MetaWaylandBuffer *buffer)
 {
@@ -267,6 +221,9 @@ ensure_buffer_texture (MetaWaylandBuffer *buffer)
   CoglError *catch_error = NULL;
   CoglTexture *texture;
 
+  if (!buffer)
+    return;
+
   texture = COGL_TEXTURE (cogl_wayland_texture_2d_new_from_buffer (ctx,
                                                                    buffer->resource,
                                                                    &catch_error));
@@ -282,6 +239,94 @@ ensure_buffer_texture (MetaWaylandBuffer *buffer)
   buffer->height = cogl_texture_get_height (texture);
 }
 
+static void
+cursor_surface_commit (MetaWaylandSurface *surface)
+{
+  MetaWaylandBuffer *buffer = surface->pending.buffer;
+
+  if (surface->pending.newly_attached && buffer != surface->buffer_ref.buffer)
+    {
+      ensure_buffer_texture (buffer);
+      meta_wayland_buffer_reference (&surface->buffer_ref, buffer);
+    }
+
+  meta_wayland_seat_update_sprite (surface->compositor->seat);
+}
+
+static gboolean
+actor_surface_commit (MetaWaylandSurface *surface)
+{
+  MetaSurfaceActor *surface_actor = surface->surface_actor;
+  MetaWaylandBuffer *buffer = surface->pending.buffer;
+  gboolean changed = FALSE;
+
+  /* wl_surface.attach */
+  if (surface->pending.newly_attached && buffer != surface->buffer_ref.buffer)
+    {
+      ensure_buffer_texture (buffer);
+      meta_wayland_buffer_reference (&surface->buffer_ref, buffer);
+      meta_surface_actor_attach_wayland_buffer (surface_actor, buffer);
+      changed = TRUE;
+    }
+
+  surface_process_damage (surface, surface->pending.damage);
+
+  if (surface->pending.opaque_region)
+    meta_surface_actor_set_opaque_region (surface_actor, surface->pending.opaque_region);
+  if (surface->pending.input_region)
+    meta_surface_actor_set_input_region (surface_actor, surface->pending.input_region);
+
+  return changed;
+}
+
+static void
+toplevel_surface_commit (MetaWaylandSurface *surface)
+{
+  if (actor_surface_commit (surface))
+    {
+      MetaWindow *window = surface->window;
+      MetaWaylandBuffer *buffer = surface->pending.buffer;
+
+      meta_window_set_surface_mapped (window, buffer != NULL);
+      /* We resize X based surfaces according to X events */
+      if (buffer != NULL && window->client_type == META_WINDOW_CLIENT_TYPE_WAYLAND)
+        {
+          int new_width;
+          int new_height;
+
+          new_width = surface->buffer_ref.buffer->width;
+          new_height = surface->buffer_ref.buffer->height;
+          if (new_width != window->rect.width ||
+              new_height != window->rect.height ||
+              surface->pending.dx != 0 ||
+              surface->pending.dy != 0)
+            meta_window_move_resize_wayland (window, new_width, new_height,
+                                             surface->pending.dx, surface->pending.dy);
+        }
+    }
+}
+
+static void
+subsurface_surface_commit (MetaWaylandSurface *surface)
+{
+  if (actor_surface_commit (surface))
+    {
+      MetaSurfaceActor *surface_actor = surface->surface_actor;
+      MetaWaylandBuffer *buffer = surface->pending.buffer;
+      float x, y;
+
+      if (buffer != NULL)
+        clutter_actor_show (CLUTTER_ACTOR (surface_actor));
+      else
+        clutter_actor_hide (CLUTTER_ACTOR (surface_actor));
+
+      clutter_actor_get_position (CLUTTER_ACTOR (surface_actor), &x, &y);
+      x += surface->pending.dx;
+      y += surface->pending.dy;
+      clutter_actor_set_position (CLUTTER_ACTOR (surface_actor), x, y);
+    }
+}
+
 static void
 meta_wayland_surface_commit (struct wl_client *client,
                              struct wl_resource *resource)
@@ -295,85 +340,19 @@ meta_wayland_surface_commit (struct wl_client *client,
 
   compositor = surface->compositor;
 
-  /* wl_surface.attach */
-  if (surface->pending.newly_attached &&
-      surface->buffer_ref.buffer != surface->pending.buffer)
-    {
-      MetaWaylandBuffer *buffer = surface->pending.buffer;
-
-      if (buffer)
-        {
-          /* Note: we set this before informing any window-actor since the
-           * window actor will expect to find the new buffer within the
-           * surface. */
-          ensure_buffer_texture (buffer);
-          meta_wayland_buffer_reference (&surface->buffer_ref, buffer);
-        }
-    }
-
-  surface_ensure_window (surface);
-
   if (surface == compositor->seat->sprite)
-    meta_wayland_seat_update_sprite (compositor->seat);
+    cursor_surface_commit (surface);
   else if (surface->window)
-    {
-      MetaWindow *window = surface->window;
-
-      if (surface->pending.buffer)
-	{
-	  MetaWaylandBuffer *buffer = surface->pending.buffer;
-	  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-
-	  meta_window_actor_attach_wayland_buffer (window_actor, buffer);
-	}
-
-      /* We resize X based surfaces according to X events */
-      if (window->client_type == META_WINDOW_CLIENT_TYPE_WAYLAND)
-	{
-	  int new_width;
-	  int new_height;
-
-	  new_width = surface->buffer_ref.buffer->width;
-	  new_height = surface->buffer_ref.buffer->height;
-	  if (new_width != window->rect.width ||
-	      new_height != window->rect.height ||
-	      surface->pending.dx != 0 ||
-	      surface->pending.dy != 0)
-	    meta_window_move_resize_wayland (surface->window, new_width, new_height,
-					     surface->pending.dx, surface->pending.dy);
-	}
-
-      meta_window_set_opaque_region (surface->window, surface->pending.opaque_region);
-      meta_window_set_input_region (surface->window, surface->pending.input_region);
-      surface_process_damage (surface, surface->pending.damage);
-
-      meta_window_set_title (surface->window, surface->pending.title);
-      g_clear_pointer (&surface->pending.title, g_free);
-
-      meta_window_set_wm_class (surface->window, surface->pending.app_id, surface->pending.app_id);
-      g_clear_pointer (&surface->pending.app_id, g_free);
-
-      meta_window_set_gtk_dbus_properties (surface->window,
-                                           surface->pending.gtk_application_id,
-                                           surface->pending.gtk_unique_bus_name,
-                                           surface->pending.gtk_app_menu_path,
-                                           surface->pending.gtk_menubar_path,
-                                           surface->pending.gtk_application_object_path,
-                                           surface->pending.gtk_window_object_path);
-
-      g_clear_pointer (&surface->pending.gtk_application_id, g_free);
-      g_clear_pointer (&surface->pending.gtk_unique_bus_name, g_free);
-      g_clear_pointer (&surface->pending.gtk_app_menu_path, g_free);
-      g_clear_pointer (&surface->pending.gtk_menubar_path, g_free);
-      g_clear_pointer (&surface->pending.gtk_application_object_path, g_free);
-      g_clear_pointer (&surface->pending.gtk_window_object_path, g_free);
-    }
+    toplevel_surface_commit (surface);
+  else if (surface->subsurface.resource)
+    subsurface_surface_commit (surface);
 
   if (surface->pending.buffer)
     {
       wl_list_remove (&surface->pending.buffer_destroy_listener.link);
       surface->pending.buffer = NULL;
     }
+
   surface->pending.dx = 0;
   surface->pending.dy = 0;
   surface->pending.newly_attached = FALSE;
@@ -416,7 +395,7 @@ const struct wl_surface_interface meta_wayland_surface_interface = {
   meta_wayland_surface_set_buffer_scale
 };
 
-void
+static void
 meta_wayland_surface_free (MetaWaylandSurface *surface)
 {
   MetaWaylandCompositor *compositor = surface->compositor;
@@ -437,19 +416,37 @@ meta_wayland_surface_free (MetaWaylandSurface *surface)
 
   meta_wayland_compositor_repick (compositor);
 
-  g_assert (surface != compositor->seat->keyboard.focus);
-  if (surface == compositor->seat->pointer.focus)
-    {
-      meta_wayland_pointer_destroy_focus (&compositor->seat->pointer);
-
-      g_assert (surface != compositor->seat->pointer.focus);
-    }
+  g_object_unref (surface->surface_actor);
 
   if (surface->resource)
     wl_resource_set_user_data (surface->resource, NULL);
   g_slice_free (MetaWaylandSurface, surface);
 }
 
+static void
+unparent_actor (MetaWaylandSurface *surface)
+{
+  ClutterActor *parent_actor;
+
+  parent_actor = clutter_actor_get_parent (CLUTTER_ACTOR (surface->surface_actor));
+  clutter_actor_remove_child (parent_actor, CLUTTER_ACTOR (surface->surface_actor));
+}
+
+static void
+destroy_window (MetaWaylandSurface *surface)
+{
+  MetaDisplay *display = meta_get_display ();
+  guint32 timestamp = meta_display_get_current_time_roundtrip (display);
+
+  /* Remove our actor from the parent, so it doesn't get destroyed when
+   * the MetaWindowActor is destroyed. */
+  unparent_actor (surface);
+
+  g_assert (surface->window != NULL);
+  meta_window_unmanage (surface->window, timestamp);
+  surface->window = NULL;
+}
+
 static void
 meta_wayland_surface_resource_destroy_cb (struct wl_resource *resource)
 {
@@ -470,16 +467,10 @@ meta_wayland_surface_resource_destroy_cb (struct wl_resource *resource)
 
       /* NB: If the surface corresponds to an X window then we will be
        * sure to free the MetaWindow according to some X event. */
-      if (surface->window &&
-	  surface->window->client_type == META_WINDOW_CLIENT_TYPE_WAYLAND)
-	{
-	  MetaDisplay *display = meta_get_display ();
-	  guint32 timestamp = meta_display_get_current_time_roundtrip (display);
+      if (surface->window && surface->window->client_type == META_WINDOW_CLIENT_TYPE_WAYLAND)
+        destroy_window (surface);
 
-	  meta_window_unmanage (surface->window, timestamp);
-	}
-      else if (!surface->window)
-	meta_wayland_surface_free (surface);
+      meta_wayland_surface_free (surface);
     }
 }
 
@@ -515,29 +506,91 @@ meta_wayland_surface_create (MetaWaylandCompositor *compositor,
     surface_handle_pending_buffer_destroy;
   wl_list_init (&surface->pending.frame_callback_list);
 
+  surface->surface_actor = g_object_ref_sink (meta_surface_actor_new ());
   return surface;
 }
 
+static void
+destroy_surface_extension (MetaWaylandSurfaceExtension *extension)
+{
+  wl_list_remove (&extension->surface_destroy_listener.link);
+  extension->surface_destroy_listener.notify = NULL;
+  extension->resource = NULL;
+}
+
+static void
+extension_handle_surface_destroy (struct wl_listener *listener,
+				  void *data)
+{
+  MetaWaylandSurfaceExtension *extension = wl_container_of (listener, extension, surface_destroy_listener);
+  wl_resource_destroy (extension->resource);
+}
+
+static int
+get_resource_version (struct wl_resource *master_resource,
+                      int                 max_version)
+{
+  return MIN (max_version, wl_resource_get_version (master_resource));
+}
+
+static gboolean
+create_surface_extension (MetaWaylandSurfaceExtension *extension,
+                          struct wl_client            *client,
+                          struct wl_resource          *master_resource,
+                          struct wl_resource          *surface_resource,
+                          guint32                      id,
+                          int                          max_version,
+                          const struct wl_interface   *interface,
+                          const void                  *implementation,
+                          wl_resource_destroy_func_t   destructor)
+{
+  struct wl_resource *resource;
+
+  if (extension->resource != NULL)
+    return FALSE;
+
+  resource = wl_resource_create (client, interface, get_resource_version (master_resource, max_version), id);
+  wl_resource_set_implementation (resource, implementation, extension, destructor);
+
+  extension->resource = resource;
+  extension->surface_destroy_listener.notify = extension_handle_surface_destroy;
+  wl_resource_add_destroy_listener (surface_resource, &extension->surface_destroy_listener);
+  return TRUE;
+}
+
+static void
+xdg_surface_destructor (struct wl_resource *resource)
+{
+  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+
+  destroy_window (surface);
+  destroy_surface_extension (xdg_surface);
+}
+
 static void
 xdg_surface_destroy (struct wl_client *client,
                      struct wl_resource *resource)
 {
-  g_warning ("TODO: support xdg_surface.destroy");
+  wl_resource_destroy (resource);
 }
 
 static void
 xdg_surface_set_transient_for (struct wl_client *client,
                                struct wl_resource *resource,
-                               struct wl_resource *parent)
+                               struct wl_resource *parent_resource)
 {
-  MetaWaylandSurfaceExtension *surface_ext = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = surface_ext->surface;
+  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+  MetaWindow *transient_for = NULL;
 
-  MetaWaylandSurfaceExtension *parent_ext = wl_resource_get_user_data (parent);
-  MetaWaylandSurface *parent_surf = parent_ext->surface;
+  if (parent_resource)
+    {
+      MetaWaylandSurface *parent_surface = wl_resource_get_user_data (parent_resource);
+      transient_for = parent_surface->window;
+    }
 
-  if (surface->window && parent_surf->window)
-    meta_window_set_transient_for (surface->window, parent_surf->window);
+  meta_window_set_transient_for (surface->window, transient_for);
 }
 
 static void
@@ -545,11 +598,10 @@ xdg_surface_set_title (struct wl_client *client,
                        struct wl_resource *resource,
                        const char *title)
 {
-  MetaWaylandSurfaceExtension *extension = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = extension->surface;
+  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  g_clear_pointer (&surface->pending.title, g_free);
-  surface->pending.title = g_strdup (title);
+  meta_window_set_title (surface->window, title);
 }
 
 static void
@@ -557,11 +609,10 @@ xdg_surface_set_app_id (struct wl_client *client,
                         struct wl_resource *resource,
                         const char *app_id)
 {
-  MetaWaylandSurfaceExtension *extension = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = extension->surface;
+  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  g_clear_pointer (&surface->pending.app_id, g_free);
-  surface->pending.app_id = g_strdup (app_id);
+  meta_window_set_wm_class (surface->window, app_id, app_id);
 }
 
 static void
@@ -569,6 +620,10 @@ xdg_surface_pong (struct wl_client *client,
                   struct wl_resource *resource,
                   guint32 serial)
 {
+  MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
+
+  meta_window_pong (surface->window, serial);
 }
 
 static gboolean
@@ -578,9 +633,6 @@ begin_grab_op_on_surface (MetaWaylandSurface *surface,
 {
   MetaWindow *window = surface->window;
 
-  if (!window)
-    return FALSE;
-
   if (grab_op == META_GRAB_OP_NONE)
     return FALSE;
 
@@ -605,13 +657,14 @@ xdg_surface_move (struct wl_client *client,
 {
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
   if (seat->pointer.button_count == 0 ||
       seat->pointer.grab_serial != serial ||
-      seat->pointer.focus != xdg_surface->surface)
+      seat->pointer.focus != surface)
     return;
 
-  begin_grab_op_on_surface (xdg_surface->surface, seat, META_GRAB_OP_MOVING);
+  begin_grab_op_on_surface (surface, seat, META_GRAB_OP_MOVING);
 }
 
 static MetaGrabOp
@@ -650,13 +703,14 @@ xdg_surface_resize (struct wl_client *client,
 {
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
   if (seat->pointer.button_count == 0 ||
       seat->pointer.grab_serial != serial ||
-      seat->pointer.focus != xdg_surface->surface)
+      seat->pointer.focus != surface)
     return;
 
-  begin_grab_op_on_surface (xdg_surface->surface, seat, grab_op_for_edge (edges));
+  begin_grab_op_on_surface (surface, seat, grab_op_for_edge (edges));
 }
 
 static void
@@ -672,10 +726,9 @@ xdg_surface_set_fullscreen (struct wl_client *client,
                             struct wl_resource *resource)
 {
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = xdg_surface->surface;
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  if (surface->window)
-    meta_window_make_fullscreen (surface->window);
+  meta_window_make_fullscreen (surface->window);
 }
 
 static void
@@ -683,10 +736,9 @@ xdg_surface_unset_fullscreen (struct wl_client *client,
                               struct wl_resource *resource)
 {
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = xdg_surface->surface;
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  if (surface->window)
-    meta_window_unmake_fullscreen (surface->window);
+  meta_window_unmake_fullscreen (surface->window);
 }
 
 static void
@@ -694,10 +746,9 @@ xdg_surface_set_maximized (struct wl_client *client,
                            struct wl_resource *resource)
 {
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = xdg_surface->surface;
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  if (surface->window)
-    meta_window_maximize (surface->window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
+  meta_window_maximize (surface->window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
 }
 
 static void
@@ -705,10 +756,9 @@ xdg_surface_unset_maximized (struct wl_client *client,
                              struct wl_resource *resource)
 {
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = xdg_surface->surface;
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  if (surface->window)
-    meta_window_unmaximize (surface->window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
+  meta_window_unmaximize (surface->window, META_MAXIMIZE_HORIZONTAL | META_MAXIMIZE_VERTICAL);
 }
 
 static void
@@ -716,10 +766,9 @@ xdg_surface_set_minimized (struct wl_client *client,
                            struct wl_resource *resource)
 {
   MetaWaylandSurfaceExtension *xdg_surface = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = xdg_surface->surface;
+  MetaWaylandSurface *surface = wl_container_of (xdg_surface, surface, xdg_surface);
 
-  if (surface->window)
-    meta_window_minimize (surface->window);
+  meta_window_minimize (surface->window);
 }
 
 static const struct xdg_surface_interface meta_wayland_xdg_surface_interface = {
@@ -738,55 +787,6 @@ static const struct xdg_surface_interface meta_wayland_xdg_surface_interface = {
   xdg_surface_set_minimized,
 };
 
-static void
-extension_handle_surface_destroy (struct wl_listener *listener,
-				  void *data)
-{
-  MetaWaylandSurfaceExtension *extension =
-    wl_container_of (listener, extension, surface_destroy_listener);
-
-  extension->surface = NULL;
-  wl_resource_destroy (extension->resource);
-}
-
-static void
-destroy_surface_extension (struct wl_resource *resource)
-{
-  MetaWaylandSurfaceExtension *extension = wl_resource_get_user_data (resource);
-
-  /* In case cleaning up a dead client destroys extension first */
-  if (extension->surface)
-    wl_list_remove (&extension->surface_destroy_listener.link);
-
-  g_free (extension);
-}
-
-static MetaWaylandSurfaceExtension *
-create_surface_extension (struct wl_client          *client,
-			  struct wl_resource        *master_resource,
-			  guint32                    id,
-			  int                        max_version,
-			  MetaWaylandSurface        *surface,
-			  const struct wl_interface *interface,
-			  const void                *implementation)
-{
-  MetaWaylandSurfaceExtension *extension;
-
-  extension = g_new0 (MetaWaylandSurfaceExtension, 1);
-
-  extension->resource = wl_resource_create (client, interface,
-					    MIN (max_version, wl_resource_get_version (master_resource)), id);
-  wl_resource_set_implementation (extension->resource, implementation,
-				  extension, destroy_surface_extension);
-
-  extension->surface = surface;
-  extension->surface_destroy_listener.notify = extension_handle_surface_destroy;
-  wl_resource_add_destroy_listener (surface->resource,
-                                    &extension->surface_destroy_listener);
-
-  return extension;
-}
-
 static void
 use_unstable_version (struct wl_client *client,
                       struct wl_resource *resource,
@@ -804,7 +804,11 @@ get_xdg_surface (struct wl_client *client,
 {
   MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
 
-  if (surface->xdg_surface)
+  if (!create_surface_extension (&surface->xdg_surface, client, surface_resource, resource, id,
+                                 META_XDG_SURFACE_VERSION,
+                                 &xdg_surface_interface,
+                                 &meta_wayland_xdg_surface_interface,
+                                 xdg_surface_destructor))
     {
       wl_resource_post_error (surface_resource,
                               WL_DISPLAY_ERROR_INVALID_OBJECT,
@@ -812,25 +816,87 @@ get_xdg_surface (struct wl_client *client,
       return;
     }
 
-  surface->xdg_surface = create_surface_extension (client, resource, id,
-                                                   META_XDG_SURFACE_VERSION, surface,
-                                                   &xdg_surface_interface,
-                                                   &meta_wayland_xdg_surface_interface);
+  surface->window = meta_window_new_for_wayland (meta_get_display (), surface);
 }
 
+static void
+xdg_popup_destructor (struct wl_resource *resource)
+{
+  MetaWaylandSurfaceExtension *xdg_popup = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_popup, surface, xdg_popup);
+
+  destroy_window (surface);
+  destroy_surface_extension (xdg_popup);
+}
+
+static void
+xdg_popup_destroy (struct wl_client *client,
+                   struct wl_resource *resource)
+{
+  wl_resource_destroy (resource);
+}
+
+static void
+xdg_popup_pong (struct wl_client *client,
+                struct wl_resource *resource,
+                uint32_t serial)
+{
+  MetaWaylandSurfaceExtension *xdg_popup = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (xdg_popup, surface, xdg_popup);
+
+  meta_window_pong (surface->window, serial);
+}
+
+static const struct xdg_popup_interface meta_wayland_xdg_popup_interface = {
+  xdg_popup_destroy,
+  xdg_popup_pong,
+};
+
 static void
 get_xdg_popup (struct wl_client *client,
                struct wl_resource *resource,
                uint32_t id,
-               struct wl_resource *surface,
-               struct wl_resource *parent,
-               struct wl_resource *seat,
+               struct wl_resource *surface_resource,
+               struct wl_resource *parent_resource,
+               struct wl_resource *seat_resource,
                uint32_t serial,
                int32_t x,
                int32_t y,
                uint32_t flags)
 {
-  g_warning ("TODO: support xdg_shell.get_xdg_popup");
+  MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
+  MetaWaylandSurface *parent_surf = wl_resource_get_user_data (parent_resource);
+  MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
+  MetaRectangle parent_rect;
+
+  if (parent_surf == NULL || parent_surf->window == NULL)
+    return;
+
+  meta_window_get_frame_rect (parent_surf->window, &parent_rect);
+
+  if (!create_surface_extension (&surface->xdg_popup, client, surface_resource, resource, id,
+                                 META_XDG_POPUP_VERSION,
+                                 &xdg_popup_interface,
+                                 &meta_wayland_xdg_popup_interface,
+                                 xdg_popup_destructor))
+    {
+      wl_resource_post_error (surface_resource,
+                              WL_DISPLAY_ERROR_INVALID_OBJECT,
+                              "xdg_shell::get_xdg_popup already requested");
+      return;
+    }
+
+  surface->window = meta_window_new_for_wayland (meta_get_display (), surface);
+  surface->window->rect.x = parent_rect.x + x;
+  surface->window->rect.y = parent_rect.y + y;
+  surface->window->showing_for_first_time = FALSE;
+  surface->window->placed = TRUE;
+  meta_window_set_transient_for (surface->window, parent_surf->window);
+
+  surface->window->type = META_WINDOW_DROPDOWN_MENU;
+  meta_window_type_changed (surface->window);
+
+  meta_wayland_pointer_start_popup_grab (&seat->pointer, surface);
 }
 
 static const struct xdg_shell_interface meta_wayland_xdg_shell_interface = {
@@ -852,6 +918,15 @@ bind_xdg_shell (struct wl_client *client,
   wl_resource_set_implementation (resource, &meta_wayland_xdg_shell_interface, data, NULL);
 }
 
+static void
+gtk_surface_destructor (struct wl_resource *resource)
+{
+  MetaWaylandSurfaceExtension *gtk_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (gtk_surface, surface, gtk_surface);
+
+  destroy_surface_extension (gtk_surface);
+}
+
 static void
 set_dbus_properties (struct wl_client   *client,
 		     struct wl_resource *resource,
@@ -862,29 +937,16 @@ set_dbus_properties (struct wl_client   *client,
 		     const char         *application_object_path,
 		     const char         *unique_bus_name)
 {
-  MetaWaylandSurfaceExtension *extension = wl_resource_get_user_data (resource);
-  MetaWaylandSurface *surface = extension->surface;
+  MetaWaylandSurfaceExtension *gtk_surface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (gtk_surface, surface, gtk_surface);
 
-  if (surface == NULL)
-    {
-      wl_resource_post_error (resource,
-                              WL_DISPLAY_ERROR_INVALID_OBJECT,
-                              "object is not associated with a toplevel surface");
-      return;
-    }
-
-  g_clear_pointer (&surface->pending.gtk_application_id, g_free);
-  surface->pending.gtk_application_id = g_strdup (application_id);
-  g_clear_pointer (&surface->pending.gtk_unique_bus_name, g_free);
-  surface->pending.gtk_unique_bus_name = g_strdup (unique_bus_name);
-  g_clear_pointer (&surface->pending.gtk_app_menu_path, g_free);
-  surface->pending.gtk_app_menu_path = g_strdup (app_menu_path);
-  g_clear_pointer (&surface->pending.gtk_menubar_path, g_free);
-  surface->pending.gtk_menubar_path = g_strdup (menubar_path);
-  g_clear_pointer (&surface->pending.gtk_application_object_path, g_free);
-  surface->pending.gtk_application_object_path = g_strdup (application_object_path);
-  g_clear_pointer (&surface->pending.gtk_window_object_path, g_free);
-  surface->pending.gtk_window_object_path = g_strdup (window_object_path);
+  meta_window_set_gtk_dbus_properties (surface->window,
+                                       application_id,
+                                       unique_bus_name,
+                                       app_menu_path,
+                                       menubar_path,
+                                       application_object_path,
+                                       window_object_path);
 }
 
 static const struct gtk_surface_interface meta_wayland_gtk_surface_interface = {
@@ -899,18 +961,17 @@ get_gtk_surface (struct wl_client *client,
 {
   MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
 
-  if (surface->gtk_surface)
+  if (!create_surface_extension (&surface->gtk_surface, client, surface_resource, resource, id,
+                                 META_GTK_SURFACE_VERSION,
+                                 &gtk_surface_interface,
+                                 &meta_wayland_gtk_surface_interface,
+                                 gtk_surface_destructor))
     {
       wl_resource_post_error (surface_resource,
                               WL_DISPLAY_ERROR_INVALID_OBJECT,
                               "gtk_shell::get_gtk_surface already requested");
       return;
     }
-
-  surface->gtk_surface = create_surface_extension (client, resource, id,
-                                                   META_GTK_SURFACE_VERSION, surface,
-                                                   &gtk_surface_interface,
-                                                   &meta_wayland_gtk_surface_interface);
 }
 
 static const struct gtk_shell_interface meta_wayland_gtk_shell_interface = {
@@ -933,6 +994,143 @@ bind_gtk_shell (struct wl_client *client,
   gtk_shell_send_capabilities (resource, GTK_SHELL_CAPABILITY_GLOBAL_APP_MENU);
 }
 
+static void
+wl_subsurface_destructor (struct wl_resource *resource)
+{
+  MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
+
+  unparent_actor (surface);
+  destroy_surface_extension (subsurface);
+}
+
+static void
+wl_subsurface_destroy (struct wl_client *client,
+                       struct wl_resource *resource)
+{
+  wl_resource_destroy (resource);
+}
+
+static void
+wl_subsurface_set_position (struct wl_client *client,
+                            struct wl_resource *resource,
+                            int32_t x,
+                            int32_t y)
+{
+  MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
+
+  clutter_actor_set_position (CLUTTER_ACTOR (surface->surface_actor), x, y);
+}
+
+static void
+wl_subsurface_place_above (struct wl_client *client,
+                           struct wl_resource *resource,
+                           struct wl_resource *sibling_resource)
+{
+  ClutterActor *parent_actor;
+  MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
+  MetaWaylandSurface *sibling = wl_resource_get_user_data (sibling_resource);
+
+  parent_actor = clutter_actor_get_parent (CLUTTER_ACTOR (surface->surface_actor));
+
+  clutter_actor_set_child_above_sibling (parent_actor,
+                                         CLUTTER_ACTOR (surface->surface_actor),
+                                         CLUTTER_ACTOR (sibling->surface_actor));
+}
+
+static void
+wl_subsurface_place_below (struct wl_client *client,
+                           struct wl_resource *resource,
+                           struct wl_resource *sibling_resource)
+{
+  ClutterActor *parent_actor;
+  MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
+  MetaWaylandSurface *sibling = wl_resource_get_user_data (sibling_resource);
+
+  parent_actor = clutter_actor_get_parent (CLUTTER_ACTOR (surface->surface_actor));
+
+  clutter_actor_set_child_below_sibling (parent_actor,
+                                         CLUTTER_ACTOR (surface->surface_actor),
+                                         CLUTTER_ACTOR (sibling->surface_actor));
+}
+
+static void
+wl_subsurface_set_sync (struct wl_client *client,
+                        struct wl_resource *resource)
+{
+  g_warning ("TODO: support wl_subsurface.set_sync");
+}
+
+static void
+wl_subsurface_set_desync (struct wl_client *client,
+                          struct wl_resource *resource)
+{
+  g_warning ("TODO: support wl_subsurface.set_desync");
+}
+
+static const struct wl_subsurface_interface meta_wayland_subsurface_interface = {
+  wl_subsurface_destroy,
+  wl_subsurface_set_position,
+  wl_subsurface_place_above,
+  wl_subsurface_place_below,
+  wl_subsurface_set_sync,
+  wl_subsurface_set_desync,
+};
+
+static void
+wl_subcompositor_destroy (struct wl_client *client,
+                          struct wl_resource *resource)
+{
+  wl_resource_destroy (resource);
+}
+
+static void
+wl_subcompositor_get_subsurface (struct wl_client *client,
+                                 struct wl_resource *resource,
+                                 guint32 id,
+                                 struct wl_resource *surface_resource,
+                                 struct wl_resource *parent_resource)
+{
+  MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
+  MetaWaylandSurface *parent = wl_resource_get_user_data (parent_resource);
+
+  if (!create_surface_extension (&surface->subsurface, client, surface_resource, resource, id,
+                                 META_GTK_SURFACE_VERSION,
+                                 &wl_subsurface_interface,
+                                 &meta_wayland_subsurface_interface,
+                                 wl_subsurface_destructor))
+    {
+      wl_resource_post_error (surface_resource,
+                              WL_DISPLAY_ERROR_INVALID_OBJECT,
+                              "wl_subcompositor::get_subsurface already requested");
+      return;
+    }
+
+  clutter_actor_add_child (CLUTTER_ACTOR (parent->surface_actor),
+                           CLUTTER_ACTOR (surface->surface_actor));
+}
+
+static const struct wl_subcompositor_interface meta_wayland_subcompositor_interface = {
+  wl_subcompositor_destroy,
+  wl_subcompositor_get_subsurface,
+};
+
+static void
+bind_subcompositor (struct wl_client *client,
+                    void             *data,
+                    guint32           version,
+                    guint32           id)
+{
+  struct wl_resource *resource;
+
+  resource = wl_resource_create (client, &wl_subcompositor_interface,
+				 MIN (META_WL_SUBCOMPOSITOR_VERSION, version), id);
+  wl_resource_set_implementation (resource, &meta_wayland_subcompositor_interface, data, NULL);
+}
+
 void
 meta_wayland_init_shell (MetaWaylandCompositor *compositor)
 {
@@ -947,6 +1145,12 @@ meta_wayland_init_shell (MetaWaylandCompositor *compositor)
 			META_GTK_SHELL_VERSION,
 			compositor, bind_gtk_shell) == NULL)
     g_error ("Failed to register a global gtk-shell object");
+
+  if (wl_global_create (compositor->wayland_display,
+                        &wl_subcompositor_interface,
+                        META_WL_SUBCOMPOSITOR_VERSION,
+                        compositor, bind_subcompositor) == NULL)
+    g_error ("Failed to register a global wl-subcompositor object");
 }
 
 void
@@ -955,22 +1159,31 @@ meta_wayland_surface_configure_notify (MetaWaylandSurface *surface,
 				       int                 new_height,
 				       int                 edges)
 {
-  if (surface->xdg_surface)
-    xdg_surface_send_configure (surface->xdg_surface->resource,
-                                edges, new_width, new_height,
-                                0, 0 /* XXX: support this */);
+  if (surface->xdg_surface.resource)
+    xdg_surface_send_configure (surface->xdg_surface.resource,
+                                edges, new_width, new_height);
 }
 
 void
 meta_wayland_surface_focused_set (MetaWaylandSurface *surface)
 {
-  if (surface->xdg_surface)
-    xdg_surface_send_focused_set (surface->xdg_surface->resource);
+  if (surface->xdg_surface.resource)
+    xdg_surface_send_focused_set (surface->xdg_surface.resource);
 }
 
 void
 meta_wayland_surface_focused_unset (MetaWaylandSurface *surface)
 {
-  if (surface->xdg_surface)
-    xdg_surface_send_focused_unset (surface->xdg_surface->resource);
+  if (surface->xdg_surface.resource)
+    xdg_surface_send_focused_unset (surface->xdg_surface.resource);
+}
+
+void
+meta_wayland_surface_ping (MetaWaylandSurface *surface,
+                           guint32             timestamp)
+{
+  if (surface->xdg_surface.resource)
+    xdg_surface_send_ping (surface->xdg_surface.resource, timestamp);
+  else if (surface->xdg_popup.resource)
+    xdg_popup_send_ping (surface->xdg_popup.resource, timestamp);
 }
