@@ -497,11 +497,9 @@ const static struct wl_compositor_interface meta_wayland_compositor_interface =
   meta_wayland_compositor_create_region
 };
 
-static void
-paint_finished_cb (ClutterActor *self, void *user_data)
+void
+meta_wayland_compositor_paint_finished (MetaWaylandCompositor *compositor)
 {
-  MetaWaylandCompositor *compositor = user_data;
-
   while (!wl_list_empty (&compositor->frame_callbacks))
     {
       MetaWaylandFrameCallback *callback =
@@ -616,6 +614,15 @@ set_gnome_env (const char *name,
     }
 }
 
+static void
+meta_wayland_log_func (const char *fmt,
+                       va_list     arg)
+{
+  char *str = g_strdup_vprintf (fmt, arg);
+  g_warning ("WL: %s", str);
+  g_free (str);
+}
+
 void
 meta_wayland_init (void)
 {
@@ -633,6 +640,7 @@ meta_wayland_init (void)
     g_error ("failed to create wayland display");
 
   wl_display_init_shm (compositor->wayland_display);
+  wl_log_set_handler_server(meta_wayland_log_func);
 
   wl_list_init (&compositor->frame_callbacks);
 
@@ -698,8 +706,6 @@ meta_wayland_init (void)
   compositor->outputs = meta_wayland_compositor_update_outputs (compositor, monitors);
 
   compositor->stage = meta_wayland_stage_new ();
-  g_signal_connect_after (compositor->stage, "paint",
-                          G_CALLBACK (paint_finished_cb), compositor);
   g_signal_connect (compositor->stage, "destroy",
                     G_CALLBACK (stage_destroy_cb), NULL);
 
